import uuid
from enum import Enum
from pydantic import BaseModel, Field
from typing import Annotated

from mpai_cae_arp.audio.standards import EqualizationStandard, SpeedStandard


class IrregularityType(Enum):
    BRANDS_ON_TAPE = "b"
    SPLICE = "sp"
    START_OF_TAPE = "sot"
    ENDS_OF_TAPE = "eot"
    DAMAGED_TAPE = "da"
    DIRT = "di"
    MARKS = "m"
    SHADOWS = "s"
    WOW_AND_FLUTTER = "wf"
    PLAY_PAUSE_STOP = "pps"
    SPEED = "ssv"
    EQUALIZATION = "esv"
    SPEED_AND_EQUALIZATION = "ssv"
    BACKWARD = "sb"


class Source(Enum):
    AUDIO = "a"
    VIDEO = "v"
    BOTH = "b"


class IrregularityProperties(BaseModel):
    reading_speed: SpeedStandard
    reading_equalisation: EqualizationStandard
    writing_speed: SpeedStandard
    writing_equalisation: EqualizationStandard

    @staticmethod
    def from_json(json_property: dict):
        return IrregularityProperties(
            reading_speed=SpeedStandard(json_property["ReadingSpeedStandard"]),
            reading_equalisation=EqualizationStandard(
                json_property["ReadingEqualisationStandard"]),
            writing_speed=SpeedStandard(json_property["WritingSpeedStandard"]),
            writing_equalisation=EqualizationStandard(
                json_property["WritingEqualisationStandard"]))

    def to_json(self):
        return {
            "ReadingSpeedStandard": self.reading_speed.value,
            "ReadingEqualisationStandard": self.reading_equalisation.value,
            "WritingSpeedStandard": self.writing_speed.value,
            "WritingEqualisationStandard": self.writing_equalisation.value,
        }


class Irregularity(BaseModel):
    irregularity_ID: uuid.UUID
    source: Source
    time_label: str
    irregularity_type: IrregularityType = None
    irregularity_properties: IrregularityProperties | None = None
    image_URI: str | None = None
    audio_block_URI: str | None = None

    @staticmethod
    def from_json(json_irreg: dict):

        properties = None
        if json_irreg.get("IrregularityProperties") is not None:
            properties = IrregularityProperties.from_json(
                json_irreg["IrregularityProperties"])

        raw_irreg_type = json_irreg.get("IrregularityType")
        irregularity_type = None
        if raw_irreg_type is not None:
            if raw_irreg_type is not (
                    IrregularityType.SPEED.value
                    or IrregularityType.SPEED_AND_EQUALIZATION.value):
                irregularity_type = IrregularityType(raw_irreg_type)
            else:
                if properties.reading_equalisation != properties.writing_equalisation:
                    irregularity_type = IrregularityType.SPEED_AND_EQUALIZATION
                else:
                    irregularity_type = IrregularityType.SPEED

        return Irregularity(irregularity_ID=uuid.UUID(
            json_irreg["IrregularityID"]),
                            source=Source(json_irreg["Source"]),
                            time_label=json_irreg["TimeLabel"],
                            irregularity_type=irregularity_type,
                            irregularity_properties=properties,
                            image_URI=json_irreg.get("ImageURI"),
                            audio_block_URI=json_irreg.get("AudioBlockURI"))

    def to_json(self):
        dictionary = {
            "IrregularityID": str(self.irregularity_ID),
            "Source": self.source.value,
            "TimeLabel": self.time_label,
        }

        if self.irregularity_type:
            dictionary["IrregularityType"] = self.irregularity_type.value

        if self.image_URI:
            dictionary["ImageURI"] = self.image_URI

        if self.audio_block_URI:
            dictionary["AudioBlockURI"] = self.audio_block_URI

        if self.irregularity_properties:
            dictionary[
                "IrregularityProperties"] = self.irregularity_properties.to_json(
                )

        return dictionary


class IrregularityFile(BaseModel):
    # TODO: the offset calculation is not implemented yet, so it is set to None
    irregularities: list[Irregularity]
    offset: Annotated[int, Field(gt=0)] | None

    class Config:
        schema_extra = {
            "example": {
                "offset": 0,
                "irregularities": [
                    {
                        "irregularity_ID": "a0a0a0a0-a0a0-a0a0-a0a0-a0a0a0a0a0a0",
                        "source": "a",
                        "time_label": "00:00:00:00",
                        "irregularity_type": "b",
                        "irregularity_properties": {
                            "reading_speed": "n",
                            "reading_equalisation": "n",
                            "writing_speed": "n",
                            "writing_equalisation": "n"
                        },
                        "audio_block_URI": "https://example.com/audio.wav",
                    }
                ]
            }
        }


    def __init__(self,
                 irregularities: list[Irregularity] = [],
                 offset: int | None = None):
        self.irregularities = irregularities
        self.offset = offset

    def __eq__(self, __o: object) -> bool:
        if not isinstance(__o, IrregularityFile):
            return False

        return self.irregularities == __o.irregularities and self.offset == __o.offset

    @staticmethod
    def from_json(json_irreg: dict):
        irregularities = []

        for irreg in json_irreg["Irregularities"]:
            irregularities.append(Irregularity.from_json(irreg))

        return IrregularityFile(irregularities=irregularities,
                                offset=int(json_irreg["Offset"]))

    def to_json(self):
        dictionary = {
            "Irregularities":
            [irregularity.to_json() for irregularity in self.irregularities],
        }

        if self.offset:
            dictionary["Offset"] = self.offset

        return dictionary

    def add(self, irregularity: Irregularity):
        """Add an irregularity to the list of irregularities.

        Parameters
        ----------
        irregularity : Irregularity
            the irregularity to add

        Raises
        ------
        TypeError
            if the irregularity is not a py:class:`Irregularity` object
        """
        if not isinstance(irregularity, Irregularity):
            raise TypeError(
                "IrregularityFile.add() expects an Irregularity object")
        self.irregularities.append(irregularity)

    def join(self, other):
        """Append the irregularities of other in current irregularity file.

        Parameters
        ----------
        other : IrregularityFile
            the irregularity file you want to append at the current one
        
        Raises
        ------
        TypeError
            if other is not an instance of IrregularityFile
        """
        if not isinstance(other, IrregularityFile):
            raise TypeError("other must be an instance of IrregularityFile")
        self.irregularities += other.irregularities



import json
import yaml


def get_file_content(file_name: str, format: str) -> dict:
    """
    Parameters
    ----------
    file_name : str
        the path to the file to read
    format : str
        the format of the file to read (yaml or json)
    
    Raises
    ------
    ValueError
        if the format is not supported
    
    Returns
    -------
    dict
        the parsed content of the file
    """

    with open(file_name) as fd:
        if format == "yaml":
            content = yaml.safe_load(fd)
            return content
        elif format == "json":
            content = json.load(fd)
        else:
            raise ValueError("Format not supported")

    return content



from dataclasses import dataclass


@dataclass
class Noise:
    """
    A class to represent a noise, which is a signal that resides in a certain range of decibels, like a sort of filter.

    Args:
        label (str): a label which describes this noise
        db_max (int): the maximum value of this noise in decibels (must be less than or equal to 0)
        db_min (int): the minimum value of this noise in decibels (must be less than or equal to 0)

    Raises:
        ValueError: if ``db_max`` is less than ``db_min``, or if ``db_min`` or ``db_max`` are greater than 0

    .. topic:: More about noise classes

        Noise classes of interest are defined in :cite:`10.1162/comj_a_00487`, where we can find three main notable classes:

        1. type A, from :math:`-50dB` to :math:`-63dB`, which is noise in the middle of a recording, i.e., silence between spoken words;
        2. type B, from :math:`-63dB` to :math:`-69dB`, which is noise of the recording head without any specific input signal;
        3. type C, from :math:`-69dB` to :math:`-72dB`, which is noise coming from sections of pristine tape.
    """
    _label: str
    _db_max: int
    _db_min: int

    def __init__(self, label, db_max, db_min) -> None:
        if db_max < db_min:
            raise ValueError("db_max must be greater than db_min")
        if db_min > 0 or db_max > 0:
            raise ValueError("db_min and db_max must be negative or zero")

        self._label = label
        self._db_max = db_max
        self._db_min = db_min

    @property
    def label(self):
        return self._label

    @property
    def db_range(self):
        """Get the range of this noise."""
        return self.db_max - self.db_min

    @property
    def db_max(self):
        return self._db_max

    @property
    def db_min(self):
        return self._db_min

    def __lt__(self, __o: object) -> bool:
        if not isinstance(__o, Noise):
            raise TypeError("Cannot compare Noise with not Noise")
        return self.db_min < __o.db_min

    def __gt__(self, __o: object) -> bool:
        if not isinstance(__o, Noise):
            raise TypeError("Cannot compare Noise with not Noise")
        return self.db_max > __o.db_max

    def __le__(self, __o: object) -> bool:
        if not isinstance(__o, Noise):
            raise TypeError("Cannot compare Noise with not Noise")
        return self < __o or self == __o

    def __repr__(self):
        return f"<Noise({self.label}, {self.db_min}dB, {self.db_max}dB)>"



from pydantic import BaseModel, Field


class Contact(BaseModel):
    name: str = Field(..., description="Name of the contact person.")
    email: str = Field(..., description="Email of the contact person.")

    class Config:
        schema_extra = {
            "example": {
                "name": "John Doe",
                "email": "email@email.com"
            }
        }


class License(BaseModel):
    name: str = Field(..., description="Name of the license.")
    url: str = Field(..., description="URL of the license.")

    class Config:
        schema_extra = {
            "example": {
                "name": "MIT",
                "url": "https://opensource.org/licenses/MIT"
            }
        }


class Info(BaseModel):
    title: str = Field(..., description="The title of the API.")
    description: str = Field(..., description="A short description of the API.")
    version: str = Field(..., description="The version of the API.")
    contact: Contact = Field(
        ..., description="Contact information for the owners of the API.")
    license_info: License = Field(
        ..., description="License information for the API.")

    class Config:
        schema_extra = {
            "example": {
                "title": "API title",
                "description": "A very nice API",
                "version": "0.1.0",
                "contact": {
                    "name": "John Doe",
                    "email": "example@example.com",
                },
                "license_info": {
                    "name": "MIT",
                    "url": "https://opensource.org/licenses/MIT",
                }
            }
        }



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mpai_cae_arp', 'mpai_cae_arp.audio', 'mpai_cae_arp.types']

package_data = \
{'': ['*']}

install_requires = \
['librosa>=0.10.0.post2,<0.11.0',
 'llvmlite>=0.39.1,<0.40.0',
 'numpy==1.23.3',
 'pydantic>=1.10.7,<2.0.0',
 'pyyaml>=6.0,<7.0']

setup_kwargs = {
    'name': 'mpai-cae-arp',
    'version': '0.1.0',
    'description': 'The MPAI CAE-ARP software API',
    'long_description': '# MPAI CAE-ARP API\n\n[![LICENSE](https://img.shields.io/badge/license-GPLv3-blue.svg)](https://img.shields.io/badge/license-GPLv3-blue.svg)\n\n## Description\n\nThis package provides a set of tools for common task in MPAI CAE-ARP standard. It is usend in the official implementation of the standard and can be used as well to develop your own.\n\n## License\n\nThis software is licensed under the GPLv3 license. See the [official site](http://www.gnu.org/licenses/gpl-3.0.html) for more information.\n',
    'author': 'Matteo Spanio',
    'author_email': 'dev2@audioinnova.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.10,<4.0',
}


setup(**setup_kwargs)
